"""The ofx parser package. A package to parse OpenFlow messages.

This package is a library that parses and creates OpenFlow Messages.
It contains all implemented versions of OpenFlow protocol
"""
__version__ = '2020.2b2'
